"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * penguins-eggs-v8
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const utils_1 = tslib_1.__importDefault(require("../classes/utils"));
const pacman_1 = tslib_1.__importDefault(require("../classes/pacman"));
const settings_1 = tslib_1.__importDefault(require("../classes/settings"));
const ovary_1 = tslib_1.__importDefault(require("../classes/ovary"));
const compressors_1 = tslib_1.__importDefault(require("../classes/compressors"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const utils_2 = require("../lib/utils");
class Daddy {
    constructor() {
        this.settings = {};
    }
    async helpMe(loadDefault = false, verbose = false) {
        // Controllo configurazione
        if (!pacman_1.default.configurationCheck()) {
            console.log('- creating configuration dir...');
            await pacman_1.default.configurationInstall(verbose);
        }
        // Controllo prerequisites
        if (!(await pacman_1.default.prerequisitesCheck())) {
            console.log('- installing prerequisites...');
            await pacman_1.default.prerequisitesInstall(verbose);
        }
        // Templates
        if (!pacman_1.default.distroTemplateCheck()) {
            console.log('- distro template install...');
            await pacman_1.default.distroTemplateInstall(verbose);
        }
        // show and edit configuration
        this.settings = new settings_1.default();
        let config = {};
        if (await this.settings.load()) {
            config = this.settings.config;
            let jsonConf;
            if (!loadDefault) {
                jsonConf = await this.editConfig(config);
            }
            else {
                if (config.snapshot_prefix === '') {
                    config.snapshot_prefix = utils_1.default.snapshotPrefix(this.settings.distro.distroId, this.settings.distro.codenameId);
                    config.compression = 'fast';
                }
                jsonConf = JSON.stringify(config);
            }
            const newConf = JSON.parse(jsonConf);
            // salvo le modifiche
            config.snapshot_basename = newConf.snapshot_basename;
            config.snapshot_prefix = newConf.snapshot_prefix;
            config.user_opt = newConf.user_opt;
            config.user_opt_passwd = newConf.user_opt_passwd;
            config.root_passwd = newConf.root_passwd;
            config.theme = newConf.theme;
            /**
             * Analisi del tipo di compressione disponibile
             */
            const compressors = new compressors_1.default();
            await compressors.populate();
            config.compression = compressors.normal();
            if (newConf.compression === 'fast') {
                config.compression = compressors.fast();
            }
            else if (newConf.compression === 'max') {
                config.compression = compressors.max();
            }
            await this.settings.save(config);
            let flags = '';
            if (loadDefault) {
                await (0, utils_2.exec)(`rm ${this.settings.work_dir.path} -rf`);
                await (0, utils_2.exec)(`rm ${this.settings.config.snapshot_dir} -rf`);
            }
            else {
                // Controllo se serve il kill
                if (verbose) {
                    flags = '--verbose ';
                }
                utils_1.default.titles('kill' + flags);
                console.log(chalk_1.default.cyan('Daddy, what else did you leave for me?'));
                await this.settings.listFreeSpace();
                if (await utils_1.default.customConfirm()) {
                    await (0, utils_2.exec)(`rm ${this.settings.work_dir.path} -rf`);
                    await (0, utils_2.exec)(`rm ${this.settings.config.snapshot_dir} -rf`);
                }
                /**
                 * produce
                 */
                if (loadDefault) {
                    verbose = false;
                }
                flags += ' --' + newConf.compression;
                flags += ' --addons adapt';
                utils_1.default.titles('produce' + ' ' + flags);
                console.log(chalk_1.default.cyan('Daddy, what else did you leave for me?'));
                const myAddons = {};
                myAddons.adapt = true;
                const backup = false;
                const personal = false;
                const scriptOnly = false;
                const yolkRenew = false;
                const final = false;
                const ovary = new ovary_1.default();
                utils_1.default.warning('Produce an egg...');
                if (await ovary.fertilization(config.snapshot_prefix, config.snapshot_basename, config.theme, config.compression)) {
                    await ovary.produce(backup, personal, scriptOnly, yolkRenew, final, myAddons, verbose);
                    ovary.finished(scriptOnly);
                }
            }
        }
    }
    /**
     *
     * @param c
     */
    editConfig(c) {
        console.log(chalk_1.default.cyan('Edit and save LiveCD parameters'));
        let compressionOpt = 0;
        if (c.compression === 'xz') {
            compressionOpt = 1;
        }
        else if (c.compression === 'xz -Xbcj x86') {
            compressionOpt = 2;
        }
        if (c.snapshot_prefix === '') {
            c.snapshot_prefix = utils_1.default.snapshotPrefix(this.settings.distro.distroId, this.settings.distro.codenameId);
        }
        return new Promise(function (resolve) {
            const questions = [
                {
                    type: 'input',
                    name: 'snapshot_prefix',
                    message: 'LiveCD iso prefix: ',
                    default: c.snapshot_prefix
                },
                {
                    type: 'input',
                    name: 'snapshot_basename',
                    message: 'LiveCD iso basename: ',
                    default: c.snapshot_basename
                },
                {
                    type: 'input',
                    name: 'user_opt',
                    message: 'LiveCD user:',
                    default: c.user_opt
                },
                {
                    type: 'input',
                    name: 'user_opt_passwd',
                    message: 'LiveCD user password: ',
                    default: c.user_opt_passwd
                },
                {
                    type: 'input',
                    name: 'root_passwd',
                    message: 'LiveCD root password: ',
                    default: c.root_passwd
                },
                {
                    type: 'input',
                    name: 'theme',
                    message: 'LiveCD theme: ',
                    default: c.theme
                },
                {
                    type: 'list',
                    name: 'compression',
                    message: 'LiveCD compression: ',
                    choices: ['fast', 'normal', 'max'],
                    default: compressionOpt
                }
            ];
            inquirer_1.default.prompt(questions).then(function (options) {
                resolve(JSON.stringify(options));
            });
        });
    }
}
exports.default = Daddy;
