"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * penguins-eggs: incubator.ts
 *
 * author: Piero Proietti
 * mail: piero.proietti@gmail.com
 */
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const shelljs_1 = tslib_1.__importDefault(require("shelljs"));
const utils_1 = tslib_1.__importDefault(require("../utils"));
const jessie_1 = require("./distros/jessie");
const buster_1 = require("./distros/buster");
const focal_1 = require("./distros/focal");
const bionic_1 = require("./distros/bionic");
const rolling_1 = require("./distros/rolling");
const installer_1 = require("./installer");
const utils_2 = require("../../lib/utils");
/**
 *
 */
class Incubator {
    /**
     *
     * @param remix
     * @param distro
     * @param verbose
     */
    constructor(remix, distro, user_opt = 'live', verbose = false) {
        this.verbose = false;
        this.installer = {};
        this.installer = (0, installer_1.installer)();
        this.remix = remix;
        this.distro = distro;
        this.user_opt = user_opt;
        this.verbose = verbose;
        if (remix.branding === undefined) {
            remix.branding = 'eggs';
        }
    }
    /**
     * config
     */
    async config(release = false) {
        const verbose = true;
        const echo = utils_1.default.setEcho(verbose);
        this.createInstallerDirs();
        // DEBIAN
        switch (this.distro.codenameLikeId) {
            case 'jessie': {
                const jessie = new jessie_1.Jessie(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await jessie.create();
                break;
            }
            case 'stretch': {
                const stretch = new jessie_1.Jessie(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await stretch.create();
                break;
            }
            case 'buster': {
                const buster = new buster_1.Buster(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await buster.create();
                break;
            }
            case 'bullseye': {
                const bullseye = new buster_1.Buster(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await bullseye.create();
                break;
            }
            case 'bookworm': {
                const bookworm = new buster_1.Buster(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await bookworm.create();
                // DEVUAN
                break;
            }
            case 'beowulf': {
                const beowulf = new buster_1.Buster(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await beowulf.create();
                break;
            }
            case 'chimaera': {
                const chimaera = new buster_1.Buster(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await chimaera.create();
                break;
            }
            case 'daedalus': {
                const daedalus = new buster_1.Buster(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await daedalus.create();
                // UBUNTU
                break;
            }
            /**
             * Manjaro
             */
            case 'rolling': {
                release = true;
                const rolling = new rolling_1.Rolling(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await rolling.create();
                break;
            }
            case 'focal': {
                const focal = new focal_1.Focal(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await focal.create();
                break;
            }
            case 'groovy': {
                const groovy = new focal_1.Focal(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await groovy.create();
                break;
            }
            case 'hirsute': {
                const hirsute = new focal_1.Focal(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await hirsute.create();
                break;
            }
            case 'impish': {
                const impish = new focal_1.Focal(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await impish.create();
                break;
            }
            case 'jammy': {
                const jammy = new focal_1.Focal(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await jammy.create();
                break;
            }
            case 'bionic': {
                const bionic = new bionic_1.Bionic(this.installer, this.remix, this.distro, release, this.user_opt, this.verbose);
                await bionic.create();
                break;
            }
            // No default
        }
        this.createBranding();
    }
    /**
     *
     */
    createInstallerDirs() {
        if (this.installer.name !== 'calamares') {
            // Remove krill configuration and multiarc if present
            try {
                shelljs_1.default.exec('rm ' + this.installer.configuration + ' -rf');
            }
            catch (error) {
                console.log('error: ' + error + ' removing ' + this.installer.configuration + ' -rf');
            }
            try {
                shelljs_1.default.exec('rm ' + this.installer.multiarch + ' -rf');
            }
            catch (error) {
                console.log('error: ' + error + ' removing ' + this.installer.multiarch + ' -rf');
            }
        }
        // rootConfiguration krill calamares
        if (!node_fs_1.default.existsSync(this.installer.configuration)) {
            try {
                node_fs_1.default.mkdirSync(this.installer.configuration);
            }
            catch (error) {
                console.log('error: ' + error + ' creating ' + this.installer.configuration);
            }
        }
        if (!node_fs_1.default.existsSync(this.installer.configuration + 'branding')) {
            try {
                node_fs_1.default.mkdirSync(this.installer.configuration + 'branding');
            }
            catch (error) {
                console.log('error: ' + error + ' creating ' + this.installer.configuration + 'branding');
            }
        }
        if (!node_fs_1.default.existsSync(this.installer.configuration + 'branding/eggs')) {
            try {
                node_fs_1.default.mkdirSync(this.installer.configuration + 'branding/eggs');
            }
            catch (error) {
                console.log('error: ' + error + ' creating ' + this.installer.configuration + 'branding/eggs');
            }
        }
        if (!node_fs_1.default.existsSync(this.installer.configuration + 'modules')) {
            try {
                node_fs_1.default.mkdirSync(this.installer.configuration + 'modules');
            }
            catch (error) {
                console.log('error: ' + error + ' creating ' + this.installer.configuration + 'modules');
            }
        }
        if (!node_fs_1.default.existsSync(this.installer.multiarch)) {
            try {
                node_fs_1.default.mkdirSync(this.installer.multiarch);
            }
            catch (error) {
                console.log('error: ' + error + ' creating ' + this.installer.multiarch);
            }
        }
        if (!node_fs_1.default.existsSync(this.installer.multiarchModules)) {
            try {
                node_fs_1.default.mkdirSync(this.installer.multiarchModules);
            }
            catch (error) {
                console.log('error: ' + error + ' creating ' + this.installer.multiarchModules);
            }
        }
        /**
         * ADDONS (only for calamares)
         */
        if (this.installer.name === 'calamares') {
            const calamaresBranding = node_path_1.default.resolve(__dirname, `../../../addons/${this.remix.branding}/theme/calamares/branding`);
            if (node_fs_1.default.existsSync(calamaresBranding)) {
                if (!node_fs_1.default.existsSync(this.installer.configuration + `branding/${this.remix.branding}`)) {
                    try {
                        node_fs_1.default.mkdirSync(this.installer.configuration + `branding/${this.remix.branding}`);
                    }
                    catch (error) {
                        console.log('error: ' + error + ' creating ' + this.installer.configuration + `branding/${this.remix.branding}`);
                    }
                }
                shelljs_1.default.cp(calamaresBranding + '/*', this.installer.configuration + `branding/${this.remix.branding}/`);
            }
            else {
                console.log(`${calamaresBranding} not found!`);
                process.exit();
            }
            const calamaresIcon = node_path_1.default.resolve(__dirname, `../../../addons/${this.remix.branding}/theme/artwork/install-debian.png`);
            if (node_fs_1.default.existsSync(calamaresIcon)) {
                shelljs_1.default.cp(calamaresIcon, '/usr/share/icons/');
            }
            else {
                console.log(`${calamaresIcon} not found!`);
                process.exit();
            }
            const calamaresLauncher = node_path_1.default.resolve(__dirname, `../../../addons/${this.remix.branding}/theme/applications/install-debian.desktop`);
            if (node_fs_1.default.existsSync(calamaresLauncher)) {
                shelljs_1.default.cp(calamaresLauncher, '/usr/share/applications/');
            }
            else {
                console.log(`${calamaresLauncher} not found!`);
                process.exit();
            }
            // script di avvio
            shelljs_1.default.cp(node_path_1.default.resolve(__dirname, '../../../assets/calamares/install-debian'), '/sbin/install-debian');
        }
    }
    /**
     *
     */
    createBranding() {
        const branding = require('./branding').branding;
        const dir = this.installer.configuration + 'branding/' + this.remix.branding + '/';
        if (!node_fs_1.default.existsSync(dir)) {
            shelljs_1.default.exec(dir + ' -p');
        }
        const file = dir + 'branding.desc';
        const content = branding(this.remix, this.distro, this.verbose);
        write(file, content, this.verbose);
    }
    /**
     * non dovrebbe servire
     */
    async createInstallDebian() {
        const scriptInstallDebian = require('./calamares-modules/scripts/install-debian').installDebian;
        const scriptDir = '/usr/bin/';
        const scriptFile = scriptDir + 'install-debian';
        const scriptContent = scriptInstallDebian();
        write(scriptFile, scriptContent, this.verbose);
        await (0, utils_2.exec)(`chmod +x ${scriptFile}`);
    }
}
exports.default = Incubator;
/**
 *
 * @param file
 * @param content
 * @param verbose
 */
function write(file, content, verbose = false) {
    if (verbose) {
        console.log(`calamares: create ${file}`);
    }
    node_fs_1.default.writeFileSync(file, content, 'utf8');
}
