/**
 * penguins-eggs-v8
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
import { IRemix, IDistro } from '../interfaces';
/**
 * Utils: general porpouse utils
 * @remarks all the utilities
 */
export default class Pacman {
    static debs4calamares: string[];
    distro: IDistro;
    remix: IRemix;
    /**
     *
     * @returns
     */
    static distro(): IDistro;
    static whichGrubIsInstalled(): string;
    /**
     * check if it's installed xorg
     * @returns true if xorg is installed
     */
    static isInstalledXorg(): boolean;
    /**
     * check if it's installed wayland
     * @returns true if wayland
     */
    static isInstalledWayland(): boolean;
    /**
     * Check se la macchina ha grub adatto ad efi
     * Forse conviene spostarlo in pacman
     */
    static isUefi(): boolean;
    /**
     *
     * @returns true se GUI
     */
    static isInstalledGui(): boolean;
    /**
     * controlla se è operante xserver-xorg-core
     */
    static isRunningXorg(): boolean;
    /**
     * Constrolla se è operante wayland
     */
    static isRunningWayland(): boolean;
    /**
     * Check if the system is GUI able
     */
    static isRunningGui(): boolean;
    /**
     * Check if the system is just CLI
     */
    static isRunningCli(): boolean;
    /**
     * Crea array packages dei pacchetti da installare
     */
    static packages(remove?: boolean, verbose?: boolean): string[];
    /**
     * Restituisce VERO se i prerequisiti sono installati
     */
    static prerequisitesCheck(verbose?: boolean): Promise<boolean>;
    /**
     *
     */
    static prerequisitesInstall(verbose?: boolean): Promise<boolean>;
    /**
     * Torna verso se calamares è installato
     */
    static calamaresCheck(): Promise<boolean>;
    /**
     * Controlla se calamares è installabile
     * @returns
     */
    static isCalamaresAvailable(): boolean;
    /**
     *
     */
    static calamaresInstall(verbose?: boolean): Promise<void>;
    /**
     * calamaresPolicies
     */
    static calamaresPolicies(): Promise<void>;
    /**
     *
     */
    static calamaresRemove(verbose?: boolean): Promise<boolean>;
    /**
     * Restituisce VERO se i file di configurazione SONO presenti
     */
    static configurationCheck(): boolean;
    /**
     * Ritorna vero se machine-id è uguale
     */
    static configurationMachineNew(verbose?: boolean): Promise<boolean>;
    /**
     *
     */
    static configurationFresh(): Promise<void>;
    /**
     * Creazione del file di configurazione /etc/penguins-eggs
     */
    static configurationInstall(links?: boolean, verbose?: boolean): Promise<void>;
    /**
     * Rimozione dei file di configurazione
     */
    static configurationRemove(verbose?: boolean): Promise<void>;
    /**
     *
     * @param verbose
     */
    static autocompleteInstall(verbose?: boolean): Promise<void>;
    /**
     * Installa manPage
     */
    static manPageInstall(verbose?: boolean): Promise<void>;
    /**
     * distroTemplateCheck
     */
    static distroTemplateCheck(): boolean;
    /**
     *
     */
    static distroTemplateInstall(verbose?: boolean): Promise<void>;
    /**
     * restuisce VERO se il pacchetto è installato
     * @param debPackage
     */
    static packageIsInstalled(packageName: string): boolean;
    /**
     * restuisce VERO se il pacchetto è installato
     * @param debPackage
     */
    static packageAptAvailable(packageName: string): Promise<boolean>;
    /**
     *
     * @param debPackage
     * @returns
     */
    static packageAptLast(debPackage: string): Promise<string>;
    static packageNpmLast(packageNpm?: string): Promise<string>;
    /**
     *
     * @param cmd
     */
    static commandIsInstalled(cmd: string): Promise<boolean>;
    /**
     * Install the package packageName
     * @param packageName {string} Pacchetto Debian da installare
     * @returns {boolean} True if success
     */
    static packageInstall(packageName: string): Promise<boolean>;
    /**
     *
     * @param packages array packages
     *
     * Probabilmente da rimuovere, viene usata solo da prerequisitesRemove
     *
     */
    static filterInstalled(packages: string[]): string[];
}
