"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const fs_1 = tslib_1.__importDefault(require("fs"));
const utils_1 = tslib_1.__importDefault(require("./utils"));
const pacman_1 = tslib_1.__importDefault(require("./pacman"));
const bleach_1 = tslib_1.__importDefault(require("./bleach"));
const utils_2 = require("../lib/utils");
const shelljs_1 = tslib_1.__importDefault(require("shelljs"));
/**
 *
 */
class Yolk {
    constructor() {
        this.yolkDir = '/var/local/yolk';
        this.verbose = false;
        this.echo = {};
    }
    /**
     *
     */
    async create(verbose = false) {
        this.verbose = verbose;
        this.echo = utils_1.default.setEcho(verbose);
        utils_1.default.warning('updating system...');
        if (!pacman_1.default.commandIsInstalled('dpkg-scanpackages')) {
            process.exit(0);
        }
        let cmd = '';
        try {
            cmd = 'apt-get update --yes';
            await (0, utils_2.exec)(cmd, this.echo);
        }
        catch (error) {
            console.log(error);
            await utils_1.default.pressKeyToExit(cmd);
        }
        if (!this.yolkExists()) {
            await (0, utils_2.exec)(`mkdir ${this.yolkDir} -p`, this.echo);
            await (0, utils_2.exec)(`chown _apt:root ${this.yolkDir} -R`, this.echo);
        }
        else {
            await this.yolkClean();
        }
        // packages we need
        const packages = ['cryptsetup', 'keyutils', 'shim-signed']; // addes shim-signed
        // grub-pc just for amd64 or i386
        if (utils_1.default.machineArch() === 'amd64' || utils_1.default.machineArch() === 'i386') {
            packages.push('grub-pc');
        }
        // if not i386
        if (utils_1.default.machineArch() !== 'i386') {
            // add grub-efi-amd64
            packages.push('grub-efi-' + utils_1.default.machineArch());
            // add grub-efi-amd64-bin
            packages.push('grub-efi-' + utils_1.default.machineArch() + '-bin');
        }
        // chdir on yolkDir
        process.chdir(this.yolkDir);
        // for all packages, find dependencies and check if are not installed
        for (const package_ of packages) {
            utils_1.default.warning(`downloading package ${package_} and it's dependencies...`);
            cmd = `apt-cache depends --recurse --no-recommends --no-suggests --no-conflicts --no-breaks --no-replaces --no-enhances ${package_} | grep "^\\w" | sort -u`;
            const depends = (await (0, utils_2.exec)(cmd, { echo: false, capture: true })).data;
            await this.installDeps(depends.split('\n'));
        }
        // create Package.gz
        cmd = 'dpkg-scanpackages -h  md5,sha1,sha256 . | gzip -c > Packages.gz';
        utils_1.default.warning(cmd);
        await (0, utils_2.exec)(cmd, { echo: false, capture: true });
        // Create Release date: Sat, 14 Aug 2021 07:42:00 UTC
        const now = shelljs_1.default.exec('date -R -u').stdout.trim();
        const content = `Archive: stable\nComponent: yolk\nOrigin: penguins-eggs\nArchitecture: ${utils_1.default.machineArch()} \nDate: ${now}\n`;
        utils_1.default.warning('Writing Release');
        fs_1.default.writeFileSync('Release', content);
        // Cleaning
        utils_1.default.warning('Cleaning apt cache');
        const bleach = new bleach_1.default();
        await bleach.clean(verbose);
    }
    /**
     * if depends are not Installed
     * download depends
     * @param depends
     */
    async installDeps(depends) {
        // select for downloads only packages NOT already installed
        const toDownloads = [];
        for (const depend of depends) {
            if (depend !== '') {
                if (!pacman_1.default.packageIsInstalled(depend)) {
                    toDownloads.push(depend);
                }
            }
        }
        // now we go to downloads them
        for (const toDownload of toDownloads) {
            process.chdir(this.yolkDir);
            const cmd = `apt-get download ${toDownload}`;
            utils_1.default.warning(`- ${cmd}`);
            await (0, utils_2.exec)(cmd, this.echo);
        }
    }
    /**
    * Check if yoil exists and it's a repo
    */
    yolkExists() {
        const check = `${this.yolkDir}/Packages.gz`;
        return fs_1.default.existsSync(check);
    }
    /**
     * Svuota la repo yolk
     */
    async yolkClean() {
        await (0, utils_2.exec)(`rm ${this.yolkDir}/*`, this.echo);
    }
}
exports.default = Yolk;
