"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const core_1 = require("@oclif/core");
const utils_1 = tslib_1.__importDefault(require("../classes/utils"));
const settings_1 = tslib_1.__importDefault(require("../classes/settings"));
const incubator_1 = tslib_1.__importDefault(require("../classes/incubation/incubator"));
const pacman_1 = tslib_1.__importDefault(require("../classes/pacman"));
class Calamares extends core_1.Command {
    constructor() {
        super(...arguments);
        this.remix = {};
        this.incubator = {};
        this.settings = {};
    }
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        this.settings = new settings_1.default();
        const { flags } = await this.parse(Calamares);
        let verbose = false;
        if (flags.verbose) {
            verbose = true;
        }
        let remove = false;
        if (flags.remove) {
            remove = true;
        }
        let install = false;
        if (flags.install) {
            install = true;
        }
        let release = false;
        if (flags.release) {
            release = true;
        }
        let theme = 'eggs';
        if (flags.theme !== undefined) {
            theme = flags.theme;
        }
        console.log(`theme: ${theme}`);
        if (utils_1.default.isRoot(this.id)) {
            let installer = 'krill';
            if (pacman_1.default.isInstalledGui()) {
                installer = 'calamares';
            }
            if (installer === 'calamares') {
                if (!remove) {
                    if (await utils_1.default.customConfirm('Select yes to continue...')) {
                        /**
                         * Install calamares
                         */
                        if (install) {
                            utils_1.default.warning('Installing calamares...');
                            await pacman_1.default.calamaresInstall();
                            if (await this.settings.load()) {
                                this.settings.config.force_installer = true;
                                this.settings.save(this.settings.config);
                                await pacman_1.default.calamaresPolicies();
                            }
                        }
                        /**
                         * Configure calamares
                         */
                        if (await this.settings.load()) {
                            utils_1.default.warning('Configuring installer');
                            await this.settings.loadRemix(this.settings.config.snapshot_basename, theme);
                            this.incubator = new incubator_1.default(this.settings.remix, this.settings.distro, this.settings.config.user_opt, verbose);
                            await this.incubator.config(release);
                        }
                    }
                }
                else {
                    /**
                     * Remove calamares
                     */
                    if (await pacman_1.default.calamaresCheck()) {
                        await pacman_1.default.calamaresRemove();
                        if (await this.settings.load()) {
                            this.settings.config.force_installer = false;
                            this.settings.save(this.settings.config);
                        }
                    }
                }
            }
            else if ((await utils_1.default.customConfirm('Select yes to continue...')) && (await this.settings.load())) {
                utils_1.default.warning('Configuring krill');
                await this.settings.loadRemix(this.settings.config.snapshot_basename, theme);
                this.incubator = new incubator_1.default(this.settings.remix, this.settings.distro, this.settings.config.user_opt, verbose);
                await this.incubator.config(release);
            }
        }
    }
}
exports.default = Calamares;
Calamares.description = 'calamares or install or configure it';
Calamares.flags = {
    help: core_1.Flags.help({ char: 'h' }),
    verbose: core_1.Flags.boolean({ char: 'v' }),
    install: core_1.Flags.boolean({ char: 'i', description: "install calamares and it's dependencies" }),
    release: core_1.Flags.boolean({ char: 'r', description: "release: remove calamares and all it's dependencies after the installation" }),
    remove: core_1.Flags.boolean({ description: "remove calamares and it's dependencies" }),
    theme: core_1.Flags.string({ description: 'theme/branding for eggs and calamares' })
};
Calamares.examples = ["~$ sudo eggs calamares \ncreate/renew calamares configuration's files\n", "~$ sudo eggs calamares -i \ninstall calamares and create it's configuration's files\n"];
