"use strict";
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const utils_1 = tslib_1.__importDefault(require("../classes/utils"));
const krill_prepare_1 = tslib_1.__importDefault(require("../krill/krill-prepare"));
/**
 * Class Install
 */
class Install extends core_1.Command {
    /**
     * Execute
     */
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        const { flags } = await this.parse(Install);
        let crypted = false;
        if (flags.crypted) {
            crypted = true;
        }
        let pve = false;
        if (flags.pve) {
            pve = true;
            crypted = false;
        }
        let verbose = false;
        if (flags.verbose) {
            verbose = true;
        }
        if (utils_1.default.isRoot()) {
            if (utils_1.default.isLive()) {
                const krill = new krill_prepare_1.default();
                await krill.prepare(crypted, pve, verbose);
            }
            else {
                utils_1.default.warning('You are in an installed system!');
            }
        }
        else {
            utils_1.default.useRoot(this.id);
        }
    }
}
exports.default = Install;
Install.flags = {
    crypted: core_1.Flags.boolean({ char: 'k', description: 'crypted CLI installation' }),
    pve: core_1.Flags.boolean({ char: 'p', description: 'Proxmox VE install' }),
    help: core_1.Flags.help({ char: 'h' }),
    verbose: core_1.Flags.boolean({ char: 'v', description: 'verbose' })
};
Install.aliases = [`krill`];
Install.description = 'command-line system installer - the egg became a penguin!';
Install.examples = ['$ eggs install\nInstall the system using krill installer\n'];
