"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/* eslint-disable unicorn/no-process-exit */
/* eslint-disable no-process-exit */
/* eslint-disable no-console */
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const core_1 = require("@oclif/core");
const utils_1 = tslib_1.__importDefault(require("../classes/utils"));
const ovary_1 = tslib_1.__importDefault(require("../classes/ovary"));
const compressors_1 = tslib_1.__importDefault(require("../classes/compressors"));
const config_1 = tslib_1.__importDefault(require("./config"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const node_path_1 = tslib_1.__importDefault(require("node:path"));
class Produce extends core_1.Command {
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        const { flags } = await this.parse(Produce);
        if (utils_1.default.isRoot()) {
            /**
             * ADDONS dei vendors
             * Fino a 3
             */
            const addons = [];
            if (flags.addons) {
                const addons = flags.addons; // array
                for (let addon of addons) {
                    // se non viene specificato il vendor il default è eggs
                    if (!addon.includes('//')) {
                        addon = 'eggs/' + addon;
                    }
                    const dirAddon = node_path_1.default.resolve(__dirname, `../../addons/${addon}`);
                    if (!node_fs_1.default.existsSync(dirAddon)) {
                        console.log(dirAddon);
                        utils_1.default.warning('addon: ' + chalk_1.default.white(addon) + ' not found, terminate!');
                        process.exit();
                    }
                    const vendorAddon = addon.slice(0, Math.max(0, addon.search('/')));
                    const nameAddon = addon.substring(addon.search('/') + 1, addon.length);
                    if (nameAddon === 'theme') {
                        flags.theme = vendorAddon;
                    }
                }
            }
            /**
             * composizione dei flag
             */
            let prefix = '';
            if (flags.prefix !== undefined) {
                prefix = flags.prefix;
            }
            let basename = ''; // se vuoto viene definito da loadsetting (default nome dell'host)
            if (flags.basename !== undefined) {
                basename = flags.basename;
            }
            const compressors = new compressors_1.default();
            await compressors.populate();
            let compression = compressors.normal();
            if (flags.max) {
                compression = compressors.max();
            }
            else if (flags.fast) {
                compression = compressors.fast();
            }
            const release = flags.release;
            if (release) {
                compression = compressors.max();
            }
            const backup = flags.backup;
            const clone = flags.clone;
            const verbose = flags.verbose;
            const scriptOnly = flags.script;
            const yolkRenew = flags.yolk;
            /**
             * theme: if not defined will use eggs
             */
            let theme = 'eggs';
            if (flags.theme !== undefined) {
                theme = flags.theme;
            }
            const nointeractive = false;
            const i = await config_1.default.thatWeNeed(nointeractive, verbose, backup);
            if ((i.needApt || i.configurationInstall || i.configurationRefresh || i.distroTemplate) && (await utils_1.default.customConfirm('Select yes to continue...'))) {
                await config_1.default.install(i, verbose);
            }
            const myAddons = {};
            if (flags.addons != undefined) {
                if (flags.addons.includes('adapt')) {
                    myAddons.adapt = true;
                }
                if (flags.addons.includes('ichoice')) {
                    myAddons.ichoice = true;
                }
                if (flags.addons.includes('pve')) {
                    myAddons.pve = true;
                }
                if (flags.addons.includes('rsupport')) {
                    myAddons.rsupport = true;
                }
            }
            utils_1.default.titles(this.id + ' ' + this.argv);
            const ovary = new ovary_1.default();
            utils_1.default.warning('Produce an egg...');
            if (await ovary.fertilization(prefix, basename, theme, compression)) {
                await ovary.produce(backup, clone, scriptOnly, yolkRenew, release, myAddons, verbose);
                ovary.finished(scriptOnly);
            }
        }
        else {
            utils_1.default.useRoot(this.id);
        }
    }
}
exports.default = Produce;
Produce.flags = {
    prefix: core_1.Flags.string({ char: 'p', description: 'prefix' }),
    basename: core_1.Flags.string({ description: 'basename' }),
    backup: core_1.Flags.boolean({ char: 'b', description: 'backup mode (CRYPTED)' }),
    clone: core_1.Flags.boolean({ char: 'c', description: 'clone mode' }),
    fast: core_1.Flags.boolean({ char: 'f', description: 'fast compression' }),
    normal: core_1.Flags.boolean({ char: 'n', description: 'normal compression' }),
    max: core_1.Flags.boolean({ char: 'm', description: 'max compression' }),
    verbose: core_1.Flags.boolean({ char: 'v', description: 'verbose' }),
    yolk: core_1.Flags.boolean({ char: 'y', description: '-y force yolk renew' }),
    script: core_1.Flags.boolean({ char: 's', description: 'script mode. Generate scripts to manage iso build' }),
    help: core_1.Flags.help({ char: 'h' }),
    theme: core_1.Flags.string({ description: 'theme for livecd, calamares branding and partitions' }),
    addons: core_1.Flags.string({ multiple: true, description: 'addons to be used: adapt, ichoice, pve, rsupport' }),
    release: core_1.Flags.boolean({ description: 'release: configure GUI installer to remove eggs and calamares after installation' })
};
Produce.description = 'produce a live image from your system whithout your data';
Produce.examples = [
    '$ sudo eggs produce \nproduce an ISO called [hostname]-[arch]-YYYY-MM-DD_HHMM.iso, compressed xz (standard compression).\nIf hostname=ugo and arch=i386 ugo-x86-2020-08-25_1215.iso\n',
    '$ sudo eggs produce -v\nsame as previuos, but with --verbose output\n',
    '$ sudo eggs produce -vf\nsame as previuos, compression zstd, lz4 or gzip (depend from system capability)\n',
    '$ sudo eggs produce -vm\nsame as previuos, compression xz -Xbcj x86 (max compression, about 10%\nmore compressed)\n',
    '$ sudo eggs produce -vf --basename leo --theme debian --addons adapt \nproduce an ISO called leo-i386-2020-08-25_1215.iso compression fast,\nusing Debian theme and link to adapt\n',
    '$ sudo eggs produce -v --basename leo --theme debian --addons rsupport \nproduce an ISO called leo-i386-2020-08-25_1215.iso compression xz,\nusing Debian theme and link to dwagent\n',
    '$ sudo eggs produce -v --basename leo --rsupport \nproduce an ISO called leo-i386-2020-08-25_1215.iso compression xz, using eggs\ntheme and link to dwagent\n',
    '$ sudo eggs produce -vs --basename leo --rsupport \nproduce scripts to build an ISO as the previus example. Scripts can be found\nin /home/eggs/ovarium and you can customize all you need\n'
];
