"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const core_1 = require("@oclif/core");
const utils_1 = tslib_1.__importDefault(require("../classes/utils"));
const pacman_1 = tslib_1.__importDefault(require("../classes/pacman"));
const node_child_process_1 = require("node:child_process");
/**
 *
 */
class Remove extends core_1.Command {
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        const { flags } = await this.parse(Remove);
        let verbose = false;
        if (flags.verbose) {
            verbose = true;
        }
        if (utils_1.default.isRoot()) {
            /**
             * debian package
             */
            if (utils_1.default.isDebPackage() && (await utils_1.default.customConfirm())) {
                /**
                 * in caso di autoremove, tolgo le dipendenze di versione PRIMA di eggs, altrimenti si inchioda
                 */
                if (flags.autoremove) {
                    /**
                     * Rimozione dipendenze da versione live-config / open-infrastructure-system-config (ubuntu bionic)
                     */
                    if (pacman_1.default.packageIsInstalled('open-infrastructure-system-config')) {
                        (0, node_child_process_1.execSync)('apt-get purge open-infrastructure-system-config --yes');
                    }
                    else if (pacman_1.default.packageIsInstalled('live-config')) {
                        (0, node_child_process_1.execSync)('apt-get purge live-config --yes');
                    }
                }
                if (flags.purge) {
                    (0, node_child_process_1.execSync)('apt-get purge eggs --yes');
                }
                else {
                    (0, node_child_process_1.execSync)('apt-get remove eggs --yes');
                }
                if (flags.autoremove) {
                    (0, node_child_process_1.execSync)('apt-get autoremove --yes');
                }
            }
            /**
             * sources
             */
            if (utils_1.default.isSources() && (await utils_1.default.customConfirm())) {
                if (flags.autoremove) {
                    // await Pacman.prerequisitesRemove()
                }
                if (flags.purge) {
                    await pacman_1.default.configurationRemove();
                    // eggs completion
                    (0, node_child_process_1.execSync)('rm -f /etc/bash_completion.d/eggs.bash');
                    // manpages
                    (0, node_child_process_1.execSync)('rm -f /usr/share/man/man1/eggs.1.gz');
                }
                utils_1.default.warning("You are using  eggs as sources. I'll NOT remove it");
            }
        }
        else {
            utils_1.default.useRoot(this.id);
        }
    }
}
exports.default = Remove;
Remove.description = 'remove eggs and others stuff';
Remove.examples = ['$ sudo eggs remove \nremove eggs\n', "$ sudo eggs remove --purge \nremove eggs, eggs configurations, configuration's files\n", '$ sudo eggs remove --autoremove \nremove eggs, eggs configurations, packages dependencies\n'];
Remove.flags = {
    purge: core_1.Flags.boolean({ char: 'p', description: 'remove eggs configurations files' }),
    autoremove: core_1.Flags.boolean({ char: 'a', description: 'remove eggs packages dependencies' }),
    help: core_1.Flags.help({ char: 'h' }),
    verbose: core_1.Flags.boolean({ char: 'v', description: 'verbose' })
};
