"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ironing = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const utils_1 = tslib_1.__importDefault(require("../../classes/utils"));
const path_1 = tslib_1.__importDefault(require("path"));
const js_yaml_1 = tslib_1.__importDefault(require("js-yaml"));
const fs_1 = tslib_1.__importDefault(require("fs"));
/**
 *
 */
class Ironing extends core_1.Command {
    async run() {
        const { args, flags } = await this.parse(Ironing);
        utils_1.default.titles(this.id + ' ' + this.argv);
        let verbose = false;
        if (flags.verbose) {
            verbose = true;
        }
        const echo = utils_1.default.setEcho(verbose);
        let wardrobe = await utils_1.default.wardrobe();
        if (flags.wardrobe !== undefined) {
            wardrobe = flags.wardrobe;
        }
        if (!wardrobe.endsWith('/')) {
            wardrobe += '/';
        }
        wardrobe = `${path_1.default.resolve(process.cwd(), wardrobe)}/`;
        if (!fs_1.default.existsSync(wardrobe)) {
            utils_1.default.warning(`wardrobe: ${wardrobe} not found!`);
            process.exit();
        }
        /**
         * costume
         */
        let costume = 'costumes/colibri';
        if (this.argv['0'] !== undefined) {
            costume = this.argv['0'];
            if (costume.substring(0, 8) !== 'costumes' && costume.substring(0, 11) !== 'accessories' && costume.substring(0, 7) !== 'servers') {
                costume = `costumes/${costume}`;
            }
        }
        costume = wardrobe + costume;
        /**
         * tailorList
         */
        let tailorList = `${costume}/index.yml`;
        if (!fs_1.default.existsSync(tailorList)) {
            utils_1.default.warning(`index.yml not found in : ${costume}!`);
            process.exit();
        }
        const orig = js_yaml_1.default.load(fs_1.default.readFileSync(tailorList, 'utf-8'));
        let sorted = orig;
        sorted.name = orig.name;
        sorted.author = orig.author;
        sorted.description = orig.description;
        sorted.release = orig.release;
        sorted.distributions = orig.distributions.sort();
        if (orig.sequence !== undefined) {
            if (orig.sequence.repositories !== undefined) {
                sorted.sequence.repositories = orig.sequence.repositories;
                if (orig.sequence.repositories.sources_list !== undefined) {
                    sorted.sequence.repositories.sources_list = orig.sequence.repositories.sources_list;
                }
                if (orig.sequence.repositories.sources_list_d !== undefined) {
                    sorted.sequence.repositories.sources_list_d = orig.sequence.repositories.sources_list_d;
                }
                sorted.sequence.repositories.update = orig.sequence.repositories.update;
                sorted.sequence.repositories.upgrade = orig.sequence.repositories.upgrade;
            }
            if (orig.sequence.preinst !== undefined) {
                sorted.sequence.preinst = orig.sequence.preinst;
            }
            if (orig.sequence.dependencies !== undefined) {
                sorted.sequence.dependencies = orig.sequence.dependencies.sort();
            }
            if (orig.sequence.packages !== undefined) {
                if (Array.isArray(orig.sequence.packages)) {
                    sorted.sequence.packages = orig.sequence.packages.sort();
                }
            }
            if (orig.sequence.packages_no_install_recommends !== undefined) {
                if (Array.isArray(orig.sequence.packages_no_install_recommends)) {
                    sorted.sequence.packages_no_install_recommends = orig.sequence.packages_no_install_recommends.sort();
                }
            }
            if (orig.sequence.try_packages !== undefined) {
                if (Array.isArray(orig.sequence.try_packages)) {
                    sorted.sequence.try_packages = orig.sequence.try_packages.sort();
                }
            }
            if (orig.sequence.try_packages_no_install_recommends !== undefined) {
                if (Array.isArray(orig.sequence.try_packages_no_install_recommends)) {
                    sorted.sequence.try_packages_no_install_recommends = orig.sequence.try_packages_no_install_recommends.sort();
                }
            }
            if (orig.sequence.debs !== undefined) {
                sorted.sequence.debs = orig.sequence.debs;
            }
            if (orig.sequence.packages_python !== undefined) {
                if (Array.isArray(orig.sequence.packages_python)) {
                    sorted.sequence.packages_python = orig.sequence.packages_python.sort();
                }
            }
            if (orig.sequence.accessories !== undefined) {
                if (Array.isArray(orig.sequence.accessories)) {
                    sorted.sequence.accessories = orig.sequence.accessories.sort();
                }
            }
            if (orig.sequence.try_accessories !== undefined) {
                if (Array.isArray(orig.sequence.try_accessories)) {
                    sorted.sequence.try_accessories = orig.sequence.try_accessories.sort();
                }
            }
        }
        if (orig.customize !== undefined) {
            sorted.customize = orig.customize;
            if (orig.customize.dirs !== undefined) {
                sorted.customize.dirs = orig.customize.dirs;
            }
            if (orig.customize.hostname !== undefined) {
                sorted.customize.hostname = orig.customize.hostname;
            }
            if (orig.customize.scripts !== undefined) {
                orig.customize.scripts = orig.customize.scripts;
            }
        }
        if (orig.reboot !== undefined) {
            sorted.reboot = orig.reboot;
        }
        const ironed = `# costume: ${costume}\n---\n` + js_yaml_1.default.dump(sorted);
        console.log(ironed);
    }
}
exports.Ironing = Ironing;
Ironing.description = 'ordered show of costumes or accessories in wardrobe';
Ironing.args = [{ name: 'costume', description: 'costume', required: false }];
Ironing.flags = {
    wardrobe: core_1.Flags.string({ char: 'w', description: 'wardrobe' }),
    verbose: core_1.Flags.boolean({ char: 'v' }),
    help: core_1.Flags.help({ char: 'h' })
};
