"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const utils_1 = tslib_1.__importDefault(require("../../classes/utils"));
const path_1 = tslib_1.__importDefault(require("path"));
const js_yaml_1 = tslib_1.__importDefault(require("js-yaml"));
const fs_1 = tslib_1.__importDefault(require("fs"));
// libraries
const chalk_1 = tslib_1.__importDefault(require("chalk"));
/**
 *
 */
class List extends core_1.Command {
    async run() {
        const { args, flags } = await this.parse(List);
        let verbose = false;
        if (flags.verbose) {
            verbose = true;
        }
        const echo = utils_1.default.setEcho(verbose);
        utils_1.default.titles(this.id + ' ' + this.argv);
        /**
        * wardobe
        */
        let wardrobe = await utils_1.default.wardrobe();
        if (this.argv['0'] !== undefined) {
            wardrobe = this.argv['0'];
        }
        wardrobe = `${path_1.default.resolve(process.cwd(), wardrobe)}/`;
        if (!fs_1.default.existsSync(wardrobe)) {
            utils_1.default.warning(`wardrobe: ${wardrobe} not found!`);
            process.exit();
        }
        console.log(chalk_1.default.green(`wardrobe: `) + wardrobe);
        console.log();
        /**
        * costumes
        */
        const costumes = fs_1.default.readdirSync(`${wardrobe}costumes/`);
        console.log(chalk_1.default.green(`costumes: `));
        costumes.forEach(costume => {
            if (fs_1.default.existsSync(`${wardrobe}costumes/${costume}/index.yml`)) {
                const materials = js_yaml_1.default.load(fs_1.default.readFileSync(`${wardrobe}costumes/${costume}/index.yml`, 'utf-8'));
                console.log(chalk_1.default.cyan(costume) + ': ' + materials.description);
            }
        });
        console.log();
        /**
         * accessories
         */
        const accessories = fs_1.default.readdirSync(`${wardrobe}/accessories/`);
        console.log(chalk_1.default.green(`accessories: `));
        accessories.forEach(accessory => {
            if (fs_1.default.existsSync(`${wardrobe}/accessories/${accessory}/index.yml`)) {
                const materials = js_yaml_1.default.load(fs_1.default.readFileSync(`${wardrobe}/accessories/${accessory}/index.yml`, 'utf-8'));
                console.log(chalk_1.default.cyan(accessory) + ': ' + materials.description);
            }
        });
        console.log();
        /**
         * servers
         */
        const servers = fs_1.default.readdirSync(`${wardrobe}/servers/`);
        console.log(chalk_1.default.green(`servers: `));
        servers.forEach(server => {
            if (fs_1.default.existsSync(`${wardrobe}/servers/${server}/index.yml`)) {
                const materials = js_yaml_1.default.load(fs_1.default.readFileSync(`${wardrobe}/servers/${server}/index.yml`, 'utf-8'));
                console.log(chalk_1.default.cyan(server) + ': ' + materials.description);
            }
        });
        console.log();
    }
}
exports.default = List;
List.description = 'list costumes and accessoires in wardrobe';
List.args = [{ name: 'wardrobe', description: 'wardrobe', required: false }];
List.flags = {
    verbose: core_1.Flags.boolean({ char: 'v' }),
    help: core_1.Flags.help({ char: 'h' })
};
