"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const utils_1 = tslib_1.__importDefault(require("../../classes/utils"));
const js_yaml_1 = tslib_1.__importDefault(require("js-yaml"));
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
// libraries
const chalk_1 = tslib_1.__importDefault(require("chalk"));
/**
 *
 */
class Show extends core_1.Command {
    async run() {
        const { args, flags } = await this.parse(Show);
        let verbose = flags.verbose;
        let json = flags.json;
        const echo = utils_1.default.setEcho(verbose);
        utils_1.default.titles(this.id + ' ' + this.argv);
        let wardrobe = await utils_1.default.wardrobe();
        if (flags.wardrobe !== undefined) {
            wardrobe = flags.wardrobe;
        }
        wardrobe = `${path_1.default.resolve(process.cwd(), wardrobe)}/`;
        if (!fs_1.default.existsSync(wardrobe)) {
            utils_1.default.warning(`wardrobe: ${wardrobe} not found!`);
            process.exit();
        }
        console.log(chalk_1.default.green(`wardrobe: `) + wardrobe);
        /**
         * costume
         */
        let costume = 'costumes/colibri';
        if (this.argv['0'] !== undefined) {
            costume = this.argv['0'];
            if (costume.substring(0, 8) !== 'costumes' && costume.substring(0, 11) !== 'accessories' && costume.substring(0, 7) !== 'servers') {
                costume = `costumes/${costume}`;
            }
        }
        costume = wardrobe + costume;
        if (!fs_1.default.existsSync(costume)) {
            console.log(`costume: ${chalk_1.default.green(path_1.default.basename(costume))} not found in wardrobe: ${chalk_1.default.green(wardrobe)}`);
            process.exit();
        }
        /**
        * tailorList
        */
        let tailorList = `${costume}/index.yml`;
        if (!fs_1.default.existsSync(tailorList)) {
            utils_1.default.warning(`index.yml not found in : ${costume}!`);
            process.exit();
        }
        const materials = js_yaml_1.default.load(fs_1.default.readFileSync(tailorList, 'utf-8'));
        if (json) {
            console.log(JSON.stringify(materials, null, ' '));
        }
        else {
            console.log(js_yaml_1.default.dump(materials));
        }
    }
}
exports.default = Show;
Show.description = 'show costumes/accessories in wardrobe';
Show.args = [{ name: 'costume', description: 'costume', required: false }];
Show.flags = {
    wardrobe: core_1.Flags.string({ char: 'w', description: 'wardrobe' }),
    json: core_1.Flags.boolean({ char: 'j', description: 'output JSON' }),
    verbose: core_1.Flags.boolean({ char: 'v' }),
    help: core_1.Flags.help({ char: 'h' })
};
