/**
 * penguins-eggs: krill_sequence
 *
 * author: Piero Proietti
 * mail: piero.proietti@gmail.com
 *
 */
/**
 * Note: I'm using REACT here to get a TUI,
 *       via ink library https://github.com/vadimdemedes/ink
 */
/**
 * Ideally, I want to respect calamares way, remplementing the same (SEMPLIFIED) steps for CLI
 *
 * Phase 2 - install.
 *
 *  - partition:     partitions
 *  - mount:         mountFs, mountVfs
 *  - unpackfs:      unpackfs
 *  - _restore:      eggs syncfrom --rootdir /tmp/calamares-krill-root/
 *  - sources-yolk:  execCalamaresModule('sources-yolk')
 *  - machineid:     machineid
 *  - fstab:         fstab
 *  - locale         locale
 *  - keyboard:      keyboard
 *  - localecfg:     localecfg
 *  - users:         users
 *  - displaymanager: autologin
 *  - networkcfg:    networkcfg
 *  - hwclock:
 *  - services-systemd:
 *  - bootloader-config: execCalamaresModule('bootloader-config')
 *  - grubcfg:       grubcfg
 *  - bootloader:    bootloaded
 *  - packages:      removeInstaller
 *  - luksbootkeyfile:
 *  - plymouthcfg;
 *  - initramfscfg:  initramfscfg
 *  - initramfs:     initramfs
 *  - removeuser:    removeuser
 *  - sources-yolk-unmount: execCalamaresModule('sources-yolk-unmount')
 *  - umount:     umountVfs, this.umountFs
 */
import { IRemix, IDistro, INet } from '../interfaces';
import Settings from '../classes/settings';
import { IInstaller, IDevices } from '../interfaces';
import { ILocation, IKeyboard, IPartitions, IUsers } from '../interfaces/i-krill';
import partition from './modules/partitions';
import { mountFs, umountFs } from './modules/mount-fs';
import { mountVfs, umountVfs } from './modules/mount-vfs';
import unpackfs from './modules/unpackfs';
import machineId from './modules/machine-id';
import fstab from './modules/fstab';
import locale from './modules/locale';
import setKeyboard from './modules/set-keyboard';
import localeCfg from './modules/locale-cfg';
import addUser from './modules/add-user';
import changePassword from './modules/change-password';
import networkCfg from './modules/network-cfg';
import bootloaderConfig from './modules/bootloader-config';
import bootloaderConfigArch from './modules/bootloader-config-arch';
import bootloaderConfigUbuntu from './modules/bootloader-config-ubuntu';
import grubcfg from './modules/grubcfg';
import bootloader from './modules/bootloader';
import removeInstallerLink from './modules/remove-installer-link';
import initramfsCfg from './modules/initramfs-cfg';
import initramfs from './modules/initramfs';
import delLiveUser from './modules/del-live-user';
import setTimezone from './modules/set-timezone';
import umount from './modules/umount';
import mkfs from './modules/mkfs';
import hostname from './modules/hostname';
import hosts from './modules/hosts';
/**
 * hatching: installazione o cova!!!
 */
export default class Sequence {
    partition: typeof partition;
    mountFs: typeof mountFs;
    mountVfs: typeof mountVfs;
    unpackfs: typeof unpackfs;
    machineId: typeof machineId;
    fstab: typeof fstab;
    locale: typeof locale;
    setKeyboard: typeof setKeyboard;
    localeCfg: typeof localeCfg;
    addUser: typeof addUser;
    changePassword: typeof changePassword;
    networkCfg: typeof networkCfg;
    bootloaderConfig: typeof bootloaderConfig;
    bootloaderConfigArch: typeof bootloaderConfigArch;
    bootloaderConfigUbuntu: typeof bootloaderConfigUbuntu;
    grubcfg: typeof grubcfg;
    bootloader: typeof bootloader;
    removeInstallerLink: typeof removeInstallerLink;
    initramfsCfg: typeof initramfsCfg;
    initramfs: typeof initramfs;
    delLiveUser: typeof delLiveUser;
    umountFs: typeof umountFs;
    umountVfs: typeof umountVfs;
    setTimezone: typeof setTimezone;
    umount: typeof umount;
    mkfs: typeof mkfs;
    hostname: typeof hostname;
    hosts: typeof hosts;
    installer: IInstaller;
    installTarget: string;
    verbose: boolean;
    echo: {};
    efi: boolean;
    devices: IDevices;
    users: IUsers;
    network: INet;
    partitions: IPartitions;
    language: string;
    region: string;
    zone: string;
    keyboardModel: string;
    keyboardLayout: string;
    keyboardVariant: string;
    toNull: string;
    settings: Settings;
    remix: IRemix;
    distro: IDistro;
    luksName: string;
    luksFile: string;
    luksDevice: string;
    luksMountpoint: string;
    personalFile: string;
    /**
     * constructor
     */
    constructor(location: ILocation, keyboard: IKeyboard, partitions: IPartitions, users: IUsers, network: INet);
    /**
     * install
     * @param verbose
     * @param umount
     * @returns
     */
    install(verbose?: boolean): Promise<void>;
    /**
     *
     */
    execCalamaresModule(name: string): Promise<void>;
    /**
     * only show the result
     */
    finished(): Promise<void>;
}
